package edu.kit.ipd.descartes.adaptation.model.dmm.util;

import static org.junit.Assert.assertEquals;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.junit.Before;
import org.junit.Test;

import edu.kit.ipd.descartes.adaptation.model.dmm.PerformanceDataRepositoryModelManager;
import edu.kit.ipd.descartes.adaptation.model.dmm.util.WeightingFunctionHelper;
import edu.kit.ipd.descartes.mm.adaptation.AdaptationFactory;
import edu.kit.ipd.descartes.mm.adaptation.WeightedMetric;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.MetricType;
import edu.kit.ipd.descartes.perfdatarepo.PerfdatarepoFactory;
import edu.kit.ipd.descartes.perfdatarepo.PerformanceDataRepository;
import edu.kit.ipd.descartes.perfdatarepo.Result;

public class WeightingFunctionHelperTest {

    private static final String PERF_DATA_REPO_XMI_FILE = "./test_models/PerformanceDataRepositoryDummy.xmi";
    private static final double UTIL_WEIGHT = 2.0d;

    private PerformanceDataRepositoryModelManager perfDataModelManager = PerformanceDataRepositoryModelManager.getInstance();
    private PerformanceDataRepository perfDataRepo = null;
    private WeightingFunction function = null;

    private enum METRIC_TYPES {
        UTILIZATION, RESPONSE_TIME;
    }

    private MetricType getTypeHelper(METRIC_TYPES mType) {
        EList<MetricType> types = perfDataRepo.getMetricTypes();
        switch (mType) {
        case UTILIZATION:
            return types.get(0);
        case RESPONSE_TIME:
            return types.get(1);
        default:
            return null;
        }
    }

    @Before
    public void setUp() throws Exception {
        perfDataRepo = (PerformanceDataRepository) perfDataModelManager.load(URI.createFileURI(PERF_DATA_REPO_XMI_FILE));
        MetricType metricType = getTypeHelper(METRIC_TYPES.UTILIZATION);

        function = AdaptationFactory.eINSTANCE.createWeightingFunction();
        WeightedMetric wUtil = AdaptationFactory.eINSTANCE.createWeightedMetric();
        wUtil.setId("1");
        wUtil.setName("UtilizationMetricWeight");
        wUtil.setWeight(UTIL_WEIGHT);
        wUtil.setMetricType(metricType);
        function.getWeightedMetrics().add(wUtil);
    }

    @Test
    public void testGetWeightForMetricType() {
        double weight = 0.0;
        MetricType metricType = getTypeHelper(METRIC_TYPES.RESPONSE_TIME);
        weight = WeightingFunctionHelper.getWeightForMetricType(function, metricType);
        assertEquals(Double.NaN, weight, 0.0);

        metricType = getTypeHelper(METRIC_TYPES.UTILIZATION);
        weight = WeightingFunctionHelper.getWeightForMetricType(function, metricType);
        assertEquals(UTIL_WEIGHT, weight, 0.0);
    }

    @Test
    public void testGetValueForMetricType() {
        double testValue = 0;
        MetricType metricType = getTypeHelper(METRIC_TYPES.UTILIZATION);

        Result emptyResult = PerfdatarepoFactory.eINSTANCE.createResult();
        testValue = WeightingFunctionHelper.getValueForMetricType(metricType, emptyResult);
        assertEquals(Double.NaN, testValue, 0);

        Impact impact = perfDataRepo.getImpactHistory().get(0);
        Result before = impact.getBefore();
        testValue = WeightingFunctionHelper.getValueForMetricType(metricType, before);
        assertEquals(0.4, testValue, 0);
    }

}
